#include <stdio.h>
#include <pico/stdlib.h>

#include "pico/binary_info.h"
#include "hardware/i2c.h"

#include "ardu.h"
 

//----I2C-------
i2c_inst_t *XXi2c = i2c0;		//i2c1 para 26-27   12c0 para 0-1 ...
#define	SDA_PIN 0
#define	SCL_PIN 1
//-------------------------
void scanI2C(void);

#include "ole.h"
char miBuffer[40];

#define MILED 25

//----DHT-----------
#define PIN_DHT 15
int esperaS(char e,unsigned char entrada);
char readDHT(unsigned char entrada,int* h,int* t);
int humedad,temperatura;

//----DS18B20------
#define ONE_WIRE_PIN 22
void onewire_reset();
void onewire_write(unsigned char data);
int onewire_read();
int ds1820_read();

float temp18B;
//-------------------------


int main()
{
 int z;
	stdio_init_all();
	delay(1000);
	printf("-----------------------\n");
	printf("----Hola SENSORES------\n");
	pinMode(MILED, OUTPUT);
	
	//INIT --I2C--
	printf("SDA=%d  SDL=%d \n", SDA_PIN, SCL_PIN);
    i2c_init(i2c_default, 100000);
    gpio_set_function( SDA_PIN, GPIO_FUNC_I2C);
    gpio_set_function( SCL_PIN, GPIO_FUNC_I2C);
    gpio_pull_up( SDA_PIN);
    gpio_pull_up( SCL_PIN);
    bi_decl(bi_2pins_with_func( SDA_PIN,  SCL_PIN, GPIO_FUNC_I2C));
    
    scanI2C(); 
    OleInit();  
//---------------------------
	
	while(1)
	{
	  printf("Estado %d\n",0);
	  digitalWrite(MILED, LOW);
	  delay(500);
	  printf("Estado %d\n",1);
	  digitalWrite(MILED, HIGH);
	  xclearDisplay();
	  StringAT(0,0,"--SETA43--"); 
	  printf("_OLED_\n");

	  z=readDHT(PIN_DHT,&humedad,&temperatura); 
	  printf("Humedad=%.1f  Temperatura=%.1f C z=%d\n",(float)humedad/10,(float)temperatura/10,z);
	  sprintf(miBuffer,"DHT_Hum. %.1f %c",(float)humedad/10,37);
	  StringAT(0,2,miBuffer);	  
	  sprintf(miBuffer,"DHT_Temp. %.1f %cC",(float)temperatura/10,127);
	  StringAT(0,3,miBuffer);
	  temp18B=ds1820_read(); 
      printf("DS18b20_Temp=%0.2f ºC\n",temp18B/10);
      sprintf(miBuffer,"DS18b20_=%0.2f %cC",temp18B/10,127);
	  StringAT(0,4,miBuffer);
      
	  printf("-----------------------\n");
	  delay(2000);
	 
	}
}



//-----------I2C--------------------
void scanI2C(void)
{
 int ret;
 uint8_t rxdata;
 unsigned char addr;
 
	printf("------nI2C Bus Scan--\n");
    for ( addr= 0; addr < 255; ++addr) 
    {
     ret = i2c_read_blocking(XXi2c, addr, &rxdata, 1, false);
	 if(ret>0) printf("I2C DIRECCION=%d=%#04X %d Data=%d\n",addr,addr,ret, rxdata);	
	}
	printf("-----------------------\n");
}

//_________________DHT________________________________

int esperaS(char e,unsigned char entrada)
{
 int z;
    z=0;
    while(e==digitalRead(entrada))
     {
         delayMicroseconds(2);
         z++;
         if(z>500)return(-1);
     }
    return(z);
}

char readDHT(unsigned char entrada,int* h,int* t)
{
  unsigned char data[5];
  unsigned char n,c,b;
  int hi,ti,ci;
  
    //cli();
    pinMode(entrada, OUTPUT);
    digitalWrite(entrada,LOW);
    delayMicroseconds(2000);  
    pinMode(entrada, INPUT);

    if(-1==esperaS(1,entrada)) return(-1);
    if(-1==esperaS(0,entrada)) return(-2);
    if(-1==esperaS(1,entrada)) return(-3);
    b=c=0;
    for(n=0;n<41;n++)
      {
          data[c]<<=1;
          if(esperaS(0,entrada)<esperaS(1,entrada) ) data[c]|=1;
          b++;
          if(b>7)   { b=0;  c++; }
      }
    //sei();
    ci=data[0]+data[1]+data[2]+data[3];
    ci&=255;
    hi=data[0]; hi<<=8; hi|=data[1];
    ti=data[2]; ti<<=8; ti|=data[3];
#ifdef DEBUG    
    sprintf(buffer,"DATA=%d %d %d %d %d  suma=%d",data[0],data[1],data[2],data[3],data[4],ci);
    Serial.println(buffer);
    sprintf(buffer,"hi=%d  ti=%d",hi,ti);
    Serial.println(buffer);
#endif    
    if(ci!=data[4]) return(-10);
    *h=hi; *t=ti;
    return(0);
}

//-------------ONEWIRE-------------
void onewire_reset()
{ 
    pinMode(ONE_WIRE_PIN,OUTPUT); digitalWrite(ONE_WIRE_PIN,LOW);   
    delayMicroseconds(500); 
    digitalWrite(ONE_WIRE_PIN,HIGH);  pinMode(ONE_WIRE_PIN,INPUT); 
    delayMicroseconds(500);                 
    digitalWrite(ONE_WIRE_PIN,HIGH);  pinMode(ONE_WIRE_PIN,INPUT); 
} 


void onewire_write(unsigned char data) 
{ 
    unsigned char count; 
    for(count = 0; count < 8; ++count) 
    { 
        pinMode(ONE_WIRE_PIN,OUTPUT); digitalWrite(ONE_WIRE_PIN,LOW);
        delayMicroseconds(2);               
        if(data & 1)
          {  digitalWrite(ONE_WIRE_PIN,HIGH);  }
          else
          {  digitalWrite(ONE_WIRE_PIN,LOW);   }
        data>>=1;

        delayMicroseconds(60);                
        digitalWrite(ONE_WIRE_PIN,HIGH);  pinMode(ONE_WIRE_PIN,INPUT);
        delayMicroseconds(2);                // for more than 1us minimum. 
    } 
} 


int onewire_read() 
{ 
    unsigned char count;
    unsigned char data; 
    for(count = 0; count < 8; ++count) 
    { 
        data>>=1;
        pinMode(ONE_WIRE_PIN,OUTPUT);
        digitalWrite(ONE_WIRE_PIN,LOW);
        delayMicroseconds(2);              
        pinMode(ONE_WIRE_PIN,INPUT);
        delayMicroseconds(10);               
        if(digitalRead(ONE_WIRE_PIN)&1 )  {  data|=128;  }
        delayMicroseconds(50);  
    } 
    delayMicroseconds(100);
    return data; 
} 


int ds1820_read() 
{ 
    unsigned char busy=0, tempL, tempH;  
    long  temp;
    
    onewire_reset(); 
    onewire_write(0xCC);               
    onewire_write(0x44);   
    delay(10); 
    onewire_reset(); 
    onewire_write(0xCC);            
    onewire_write(0xBE);            
    
    tempL = onewire_read(); 
    tempH = onewire_read(); 
  
    temp=tempH; temp<<=8; temp|=tempL;
    //Serial.print(" HEX ");Serial.print(temp,HEX);
    temp=(temp*625)/1000;
  return(temp);  
}





